# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel

__all__ = [
    "ServiceCreateResponse",
    "InfraHTTPServiceConfig",
    "InfraHTTPServiceConfigHost",
    "InfraHTTPServiceConfigHostInfraIPv4Host",
    "InfraHTTPServiceConfigHostInfraIPv4HostNetwork",
    "InfraHTTPServiceConfigHostInfraIPv6Host",
    "InfraHTTPServiceConfigHostInfraIPv6HostNetwork",
    "InfraHTTPServiceConfigHostInfraDualStackHost",
    "InfraHTTPServiceConfigHostInfraDualStackHostNetwork",
    "InfraHTTPServiceConfigHostInfraHostnameHost",
    "InfraHTTPServiceConfigHostInfraHostnameHostResolverNetwork",
    "InfraHTTPServiceConfigTLSSettings",
    "InfraTCPServiceConfig",
    "InfraTCPServiceConfigHost",
    "InfraTCPServiceConfigHostInfraIPv4Host",
    "InfraTCPServiceConfigHostInfraIPv4HostNetwork",
    "InfraTCPServiceConfigHostInfraIPv6Host",
    "InfraTCPServiceConfigHostInfraIPv6HostNetwork",
    "InfraTCPServiceConfigHostInfraDualStackHost",
    "InfraTCPServiceConfigHostInfraDualStackHostNetwork",
    "InfraTCPServiceConfigHostInfraHostnameHost",
    "InfraTCPServiceConfigHostInfraHostnameHostResolverNetwork",
    "InfraTCPServiceConfigTLSSettings",
]


class InfraHTTPServiceConfigHostInfraIPv4HostNetwork(BaseModel):
    tunnel_id: str


class InfraHTTPServiceConfigHostInfraIPv4Host(BaseModel):
    ipv4: str

    network: InfraHTTPServiceConfigHostInfraIPv4HostNetwork


class InfraHTTPServiceConfigHostInfraIPv6HostNetwork(BaseModel):
    tunnel_id: str


class InfraHTTPServiceConfigHostInfraIPv6Host(BaseModel):
    ipv6: str

    network: InfraHTTPServiceConfigHostInfraIPv6HostNetwork


class InfraHTTPServiceConfigHostInfraDualStackHostNetwork(BaseModel):
    tunnel_id: str


class InfraHTTPServiceConfigHostInfraDualStackHost(BaseModel):
    ipv4: str

    ipv6: str

    network: InfraHTTPServiceConfigHostInfraDualStackHostNetwork


class InfraHTTPServiceConfigHostInfraHostnameHostResolverNetwork(BaseModel):
    tunnel_id: str

    resolver_ips: Optional[List[str]] = None


class InfraHTTPServiceConfigHostInfraHostnameHost(BaseModel):
    hostname: str

    resolver_network: InfraHTTPServiceConfigHostInfraHostnameHostResolverNetwork


InfraHTTPServiceConfigHost: TypeAlias = Union[
    InfraHTTPServiceConfigHostInfraIPv4Host,
    InfraHTTPServiceConfigHostInfraIPv6Host,
    InfraHTTPServiceConfigHostInfraDualStackHost,
    InfraHTTPServiceConfigHostInfraHostnameHost,
]


class InfraHTTPServiceConfigTLSSettings(BaseModel):
    """TLS settings for a connectivity service.

    If omitted, the default mode (`verify_full`) is used.
    """

    cert_verification_mode: str
    """TLS certificate verification mode for the connection to the origin.

    - `"verify_full"` — verify certificate chain and hostname (default)
    - `"verify_ca"` — verify certificate chain only, skip hostname check
    - `"disabled"` — do not verify the server certificate at all
    """


class InfraHTTPServiceConfig(BaseModel):
    host: InfraHTTPServiceConfigHost

    name: str

    type: Literal["tcp", "http"]

    created_at: Optional[datetime] = None

    http_port: Optional[int] = None

    https_port: Optional[int] = None

    service_id: Optional[str] = None

    tls_settings: Optional[InfraHTTPServiceConfigTLSSettings] = None
    """TLS settings for a connectivity service.

    If omitted, the default mode (`verify_full`) is used.
    """

    updated_at: Optional[datetime] = None


class InfraTCPServiceConfigHostInfraIPv4HostNetwork(BaseModel):
    tunnel_id: str


class InfraTCPServiceConfigHostInfraIPv4Host(BaseModel):
    ipv4: str

    network: InfraTCPServiceConfigHostInfraIPv4HostNetwork


class InfraTCPServiceConfigHostInfraIPv6HostNetwork(BaseModel):
    tunnel_id: str


class InfraTCPServiceConfigHostInfraIPv6Host(BaseModel):
    ipv6: str

    network: InfraTCPServiceConfigHostInfraIPv6HostNetwork


class InfraTCPServiceConfigHostInfraDualStackHostNetwork(BaseModel):
    tunnel_id: str


class InfraTCPServiceConfigHostInfraDualStackHost(BaseModel):
    ipv4: str

    ipv6: str

    network: InfraTCPServiceConfigHostInfraDualStackHostNetwork


class InfraTCPServiceConfigHostInfraHostnameHostResolverNetwork(BaseModel):
    tunnel_id: str

    resolver_ips: Optional[List[str]] = None


class InfraTCPServiceConfigHostInfraHostnameHost(BaseModel):
    hostname: str

    resolver_network: InfraTCPServiceConfigHostInfraHostnameHostResolverNetwork


InfraTCPServiceConfigHost: TypeAlias = Union[
    InfraTCPServiceConfigHostInfraIPv4Host,
    InfraTCPServiceConfigHostInfraIPv6Host,
    InfraTCPServiceConfigHostInfraDualStackHost,
    InfraTCPServiceConfigHostInfraHostnameHost,
]


class InfraTCPServiceConfigTLSSettings(BaseModel):
    """TLS settings for a connectivity service.

    If omitted, the default mode (`verify_full`) is used.
    """

    cert_verification_mode: str
    """TLS certificate verification mode for the connection to the origin.

    - `"verify_full"` — verify certificate chain and hostname (default)
    - `"verify_ca"` — verify certificate chain only, skip hostname check
    - `"disabled"` — do not verify the server certificate at all
    """


class InfraTCPServiceConfig(BaseModel):
    host: InfraTCPServiceConfigHost

    name: str

    type: Literal["tcp", "http"]

    app_protocol: Optional[Literal["postgresql", "mysql"]] = None

    created_at: Optional[datetime] = None

    service_id: Optional[str] = None

    tcp_port: Optional[int] = None

    tls_settings: Optional[InfraTCPServiceConfigTLSSettings] = None
    """TLS settings for a connectivity service.

    If omitted, the default mode (`verify_full`) is used.
    """

    updated_at: Optional[datetime] = None


ServiceCreateResponse: TypeAlias = Union[InfraHTTPServiceConfig, InfraTCPServiceConfig]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.stream import video_storage_usage_params
from ...types.stream.video_storage_usage_response import VideoStorageUsageResponse

__all__ = ["VideosResource", "AsyncVideosResource"]


class VideosResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VideosResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return VideosResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VideosResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return VideosResourceWithStreamingResponse(self)

    def storage_usage(
        self,
        *,
        account_id: str,
        creator: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[VideoStorageUsageResponse]:
        """
        Returns information about an account's storage use.

        Args:
          account_id: The account identifier tag.

          creator: A user-defined identifier for the media creator.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/stream/storage-usage", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"creator": creator}, video_storage_usage_params.VideoStorageUsageParams),
                post_parser=ResultWrapper[Optional[VideoStorageUsageResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[VideoStorageUsageResponse]], ResultWrapper[VideoStorageUsageResponse]),
        )


class AsyncVideosResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVideosResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVideosResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVideosResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncVideosResourceWithStreamingResponse(self)

    async def storage_usage(
        self,
        *,
        account_id: str,
        creator: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[VideoStorageUsageResponse]:
        """
        Returns information about an account's storage use.

        Args:
          account_id: The account identifier tag.

          creator: A user-defined identifier for the media creator.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/stream/storage-usage", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"creator": creator}, video_storage_usage_params.VideoStorageUsageParams
                ),
                post_parser=ResultWrapper[Optional[VideoStorageUsageResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[VideoStorageUsageResponse]], ResultWrapper[VideoStorageUsageResponse]),
        )


class VideosResourceWithRawResponse:
    def __init__(self, videos: VideosResource) -> None:
        self._videos = videos

        self.storage_usage = to_raw_response_wrapper(
            videos.storage_usage,
        )


class AsyncVideosResourceWithRawResponse:
    def __init__(self, videos: AsyncVideosResource) -> None:
        self._videos = videos

        self.storage_usage = async_to_raw_response_wrapper(
            videos.storage_usage,
        )


class VideosResourceWithStreamingResponse:
    def __init__(self, videos: VideosResource) -> None:
        self._videos = videos

        self.storage_usage = to_streamed_response_wrapper(
            videos.storage_usage,
        )


class AsyncVideosResourceWithStreamingResponse:
    def __init__(self, videos: AsyncVideosResource) -> None:
        self._videos = videos

        self.storage_usage = async_to_streamed_response_wrapper(
            videos.storage_usage,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ResourceSharingUpdateParams"]


class ResourceSharingUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier."""

    name: Required[str]
    """The name of the share."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["RawEditResponse"]


class RawEditResponse(BaseModel):
    id: str

    data: object



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["VariantEditParams", "Value"]


class VariantEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    value: Required[Value]
    """Value of the zone setting."""


class Value(TypedDict, total=False):
    """Value of the zone setting."""

    avif: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for avif.
    """

    bmp: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for bmp.
    """

    gif: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for gif.
    """

    jp2: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for jp2.
    """

    jpeg: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for jpeg.
    """

    jpg: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for jpg.
    """

    jpg2: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for jpg2.
    """

    png: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for png.
    """

    tif: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for tif.
    """

    tiff: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for tiff.
    """

    webp: SequenceNotStr[str]
    """
    List of strings with the MIME types of all the variants that should be served
    for webp.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .gateway_configuration_settings_param import GatewayConfigurationSettingsParam

__all__ = ["ConfigurationUpdateParams"]


class ConfigurationUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    settings: GatewayConfigurationSettingsParam
    """Specify account settings."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from .fallback_origin import (
    FallbackOriginResource,
    AsyncFallbackOriginResource,
    FallbackOriginResourceWithRawResponse,
    AsyncFallbackOriginResourceWithRawResponse,
    FallbackOriginResourceWithStreamingResponse,
    AsyncFallbackOriginResourceWithStreamingResponse,
)
from ...types.custom_hostnames import (
    custom_hostname_edit_params,
    custom_hostname_list_params,
    custom_hostname_create_params,
)
from .certificate_pack.certificate_pack import (
    CertificatePackResource,
    AsyncCertificatePackResource,
    CertificatePackResourceWithRawResponse,
    AsyncCertificatePackResourceWithRawResponse,
    CertificatePackResourceWithStreamingResponse,
    AsyncCertificatePackResourceWithStreamingResponse,
)
from ...types.custom_hostnames.custom_hostname_get_response import CustomHostnameGetResponse
from ...types.custom_hostnames.custom_hostname_edit_response import CustomHostnameEditResponse
from ...types.custom_hostnames.custom_hostname_list_response import CustomHostnameListResponse
from ...types.custom_hostnames.custom_hostname_create_response import CustomHostnameCreateResponse
from ...types.custom_hostnames.custom_hostname_delete_response import CustomHostnameDeleteResponse

__all__ = ["CustomHostnamesResource", "AsyncCustomHostnamesResource"]


class CustomHostnamesResource(SyncAPIResource):
    @cached_property
    def fallback_origin(self) -> FallbackOriginResource:
        return FallbackOriginResource(self._client)

    @cached_property
    def certificate_pack(self) -> CertificatePackResource:
        return CertificatePackResource(self._client)

    @cached_property
    def with_raw_response(self) -> CustomHostnamesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CustomHostnamesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomHostnamesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CustomHostnamesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        hostname: str,
        custom_metadata: Dict[str, str] | Omit = omit,
        ssl: custom_hostname_create_params.SSL | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomHostnameCreateResponse]:
        """
        Add a new custom hostname and request that an SSL certificate be issued for it.
        One of three validation methods—http, txt, email—should be used, with 'http'
        recommended if the CNAME is already in place (or will be soon). Specifying
        'email' will send an email to the WHOIS contacts on file for the base domain
        plus hostmaster, postmaster, webmaster, admin, administrator. If http is used
        and the domain is not already pointing to the Managed CNAME host, the PATCH
        method must be used once it is (to complete validation). Enable bundling of
        certificates using the custom_cert_bundle field. The bundling process requires
        the following condition One certificate in the bundle must use an RSA, and the
        other must use an ECDSA.

        Args:
          zone_id: Identifier.

          hostname: The custom hostname that will point to your hostname via CNAME.

          custom_metadata: Unique key/value metadata for this hostname. These are per-hostname (customer)
              settings.

          ssl: SSL properties used when creating the custom hostname.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/custom_hostnames", zone_id=zone_id),
            body=maybe_transform(
                {
                    "hostname": hostname,
                    "custom_metadata": custom_metadata,
                    "ssl": ssl,
                },
                custom_hostname_create_params.CustomHostnameCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomHostnameCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomHostnameCreateResponse]], ResultWrapper[CustomHostnameCreateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        id: str | Omit = omit,
        certificate_authority: Literal["google", "lets_encrypt", "ssl_com"] | Omit = omit,
        custom_origin_server: str | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        hostname: custom_hostname_list_params.Hostname | Omit = omit,
        hostname_status: Literal[
            "active",
            "pending",
            "active_redeploying",
            "moved",
            "pending_deletion",
            "deleted",
            "pending_blocked",
            "pending_migration",
            "pending_provisioned",
            "test_pending",
            "test_active",
            "test_active_apex",
            "test_blocked",
            "test_failed",
            "provisioned",
            "blocked",
        ]
        | Omit = omit,
        order: Literal["ssl", "ssl_status"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        ssl: Literal[0, 1] | Omit = omit,
        ssl_status: Literal[
            "initializing",
            "pending_validation",
            "deleted",
            "pending_issuance",
            "pending_deployment",
            "pending_deletion",
            "pending_expiration",
            "expired",
            "active",
            "initializing_timed_out",
            "validation_timed_out",
            "issuance_timed_out",
            "deployment_timed_out",
            "deletion_timed_out",
            "pending_cleanup",
            "staging_deployment",
            "staging_active",
            "deactivating",
            "inactive",
            "backup_issued",
            "holding_deployment",
        ]
        | Omit = omit,
        wildcard: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[CustomHostnameListResponse]:
        """
        List, search, sort, and filter all of your custom hostnames.

        Args:
          zone_id: Identifier.

          id: Hostname ID to match against. This ID was generated and returned during the
              initial custom_hostname creation. This parameter cannot be used with the
              'hostname' parameter.

          certificate_authority: Filter by the certificate authority that issued the SSL certificate.

          custom_origin_server: Filter by custom origin server name.

          direction: Direction to order hostnames.

          hostname_status: Filter by the hostname's activation status.

          order: Field to order hostnames by.

          page: Page number of paginated results.

          per_page: Number of hostnames per page.

          ssl: Whether to filter hostnames based on if they have SSL enabled.

          ssl_status: Filter by SSL certificate status.

          wildcard: Filter by whether the custom hostname is a wildcard hostname.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/custom_hostnames", zone_id=zone_id),
            page=SyncV4PagePaginationArray[CustomHostnameListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "certificate_authority": certificate_authority,
                        "custom_origin_server": custom_origin_server,
                        "direction": direction,
                        "hostname": hostname,
                        "hostname_status": hostname_status,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "ssl": ssl,
                        "ssl_status": ssl_status,
                        "wildcard": wildcard,
                    },
                    custom_hostname_list_params.CustomHostnameListParams,
                ),
            ),
            model=CustomHostnameListResponse,
        )

    def delete(
        self,
        custom_hostname_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomHostnameDeleteResponse:
        """
        Permanently deletes a custom hostname and revokes any SSL certificates that were
        issued for it. This action cannot be undone.

        Args:
          zone_id: Identifier.

          custom_hostname_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_hostname_id:
            raise ValueError(f"Expected a non-empty value for `custom_hostname_id` but received {custom_hostname_id!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/custom_hostnames/{custom_hostname_id}",
                zone_id=zone_id,
                custom_hostname_id=custom_hostname_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomHostnameDeleteResponse,
        )

    def edit(
        self,
        custom_hostname_id: str,
        *,
        zone_id: str,
        custom_metadata: Dict[str, str] | Omit = omit,
        custom_origin_server: str | Omit = omit,
        custom_origin_sni: str | Omit = omit,
        ssl: custom_hostname_edit_params.SSL | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomHostnameEditResponse]:
        """Modify SSL configuration for a custom hostname.

        When sent with SSL config that
        matches existing config, used to indicate that hostname should pass domain
        control validation (DCV). Can also be used to change validation type, e.g., from
        'http' to 'email'. Bundle an existing certificate with another certificate by
        using the "custom_cert_bundle" field. The bundling process supports combining
        certificates as long as the following condition is met. One certificate must use
        the RSA algorithm, and the other must use the ECDSA algorithm.

        Args:
          zone_id: Identifier.

          custom_hostname_id: Identifier.

          custom_metadata: Unique key/value metadata for this hostname. These are per-hostname (customer)
              settings.

          custom_origin_server: a valid hostname that’s been added to your DNS zone as an A, AAAA, or CNAME
              record.

          custom_origin_sni: A hostname that will be sent to your custom origin server as SNI for TLS
              handshake. This can be a valid subdomain of the zone or custom origin server
              name or the string ':request_host_header:' which will cause the host header in
              the request to be used as SNI. Not configurable with default/fallback origin
              server.

          ssl: SSL properties used when creating the custom hostname.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_hostname_id:
            raise ValueError(f"Expected a non-empty value for `custom_hostname_id` but received {custom_hostname_id!r}")
        return self._patch(
            path_template(
                "/zones/{zone_id}/custom_hostnames/{custom_hostname_id}",
                zone_id=zone_id,
                custom_hostname_id=custom_hostname_id,
            ),
            body=maybe_transform(
                {
                    "custom_metadata": custom_metadata,
                    "custom_origin_server": custom_origin_server,
                    "custom_origin_sni": custom_origin_sni,
                    "ssl": ssl,
                },
                custom_hostname_edit_params.CustomHostnameEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomHostnameEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomHostnameEditResponse]], ResultWrapper[CustomHostnameEditResponse]),
        )

    def get(
        self,
        custom_hostname_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomHostnameGetResponse]:
        """
        Retrieves detailed information about a specific custom hostname, including SSL
        certificate status, ownership verification, and origin configuration.

        Args:
          zone_id: Identifier.

          custom_hostname_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_hostname_id:
            raise ValueError(f"Expected a non-empty value for `custom_hostname_id` but received {custom_hostname_id!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/custom_hostnames/{custom_hostname_id}",
                zone_id=zone_id,
                custom_hostname_id=custom_hostname_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomHostnameGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomHostnameGetResponse]], ResultWrapper[CustomHostnameGetResponse]),
        )


class AsyncCustomHostnamesResource(AsyncAPIResource):
    @cached_property
    def fallback_origin(self) -> AsyncFallbackOriginResource:
        return AsyncFallbackOriginResource(self._client)

    @cached_property
    def certificate_pack(self) -> AsyncCertificatePackResource:
        return AsyncCertificatePackResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCustomHostnamesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomHostnamesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomHostnamesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCustomHostnamesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        hostname: str,
        custom_metadata: Dict[str, str] | Omit = omit,
        ssl: custom_hostname_create_params.SSL | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomHostnameCreateResponse]:
        """
        Add a new custom hostname and request that an SSL certificate be issued for it.
        One of three validation methods—http, txt, email—should be used, with 'http'
        recommended if the CNAME is already in place (or will be soon). Specifying
        'email' will send an email to the WHOIS contacts on file for the base domain
        plus hostmaster, postmaster, webmaster, admin, administrator. If http is used
        and the domain is not already pointing to the Managed CNAME host, the PATCH
        method must be used once it is (to complete validation). Enable bundling of
        certificates using the custom_cert_bundle field. The bundling process requires
        the following condition One certificate in the bundle must use an RSA, and the
        other must use an ECDSA.

        Args:
          zone_id: Identifier.

          hostname: The custom hostname that will point to your hostname via CNAME.

          custom_metadata: Unique key/value metadata for this hostname. These are per-hostname (customer)
              settings.

          ssl: SSL properties used when creating the custom hostname.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/custom_hostnames", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "hostname": hostname,
                    "custom_metadata": custom_metadata,
                    "ssl": ssl,
                },
                custom_hostname_create_params.CustomHostnameCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomHostnameCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomHostnameCreateResponse]], ResultWrapper[CustomHostnameCreateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        id: str | Omit = omit,
        certificate_authority: Literal["google", "lets_encrypt", "ssl_com"] | Omit = omit,
        custom_origin_server: str | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        hostname: custom_hostname_list_params.Hostname | Omit = omit,
        hostname_status: Literal[
            "active",
            "pending",
            "active_redeploying",
            "moved",
            "pending_deletion",
            "deleted",
            "pending_blocked",
            "pending_migration",
            "pending_provisioned",
            "test_pending",
            "test_active",
            "test_active_apex",
            "test_blocked",
            "test_failed",
            "provisioned",
            "blocked",
        ]
        | Omit = omit,
        order: Literal["ssl", "ssl_status"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        ssl: Literal[0, 1] | Omit = omit,
        ssl_status: Literal[
            "initializing",
            "pending_validation",
            "deleted",
            "pending_issuance",
            "pending_deployment",
            "pending_deletion",
            "pending_expiration",
            "expired",
            "active",
            "initializing_timed_out",
            "validation_timed_out",
            "issuance_timed_out",
            "deployment_timed_out",
            "deletion_timed_out",
            "pending_cleanup",
            "staging_deployment",
            "staging_active",
            "deactivating",
            "inactive",
            "backup_issued",
            "holding_deployment",
        ]
        | Omit = omit,
        wildcard: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomHostnameListResponse, AsyncV4PagePaginationArray[CustomHostnameListResponse]]:
        """
        List, search, sort, and filter all of your custom hostnames.

        Args:
          zone_id: Identifier.

          id: Hostname ID to match against. This ID was generated and returned during the
              initial custom_hostname creation. This parameter cannot be used with the
              'hostname' parameter.

          certificate_authority: Filter by the certificate authority that issued the SSL certificate.

          custom_origin_server: Filter by custom origin server name.

          direction: Direction to order hostnames.

          hostname_status: Filter by the hostname's activation status.

          order: Field to order hostnames by.

          page: Page number of paginated results.

          per_page: Number of hostnames per page.

          ssl: Whether to filter hostnames based on if they have SSL enabled.

          ssl_status: Filter by SSL certificate status.

          wildcard: Filter by whether the custom hostname is a wildcard hostname.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/custom_hostnames", zone_id=zone_id),
            page=AsyncV4PagePaginationArray[CustomHostnameListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "certificate_authority": certificate_authority,
                        "custom_origin_server": custom_origin_server,
                        "direction": direction,
                        "hostname": hostname,
                        "hostname_status": hostname_status,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "ssl": ssl,
                        "ssl_status": ssl_status,
                        "wildcard": wildcard,
                    },
                    custom_hostname_list_params.CustomHostnameListParams,
                ),
            ),
            model=CustomHostnameListResponse,
        )

    async def delete(
        self,
        custom_hostname_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomHostnameDeleteResponse:
        """
        Permanently deletes a custom hostname and revokes any SSL certificates that were
        issued for it. This action cannot be undone.

        Args:
          zone_id: Identifier.

          custom_hostname_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_hostname_id:
            raise ValueError(f"Expected a non-empty value for `custom_hostname_id` but received {custom_hostname_id!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/custom_hostnames/{custom_hostname_id}",
                zone_id=zone_id,
                custom_hostname_id=custom_hostname_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomHostnameDeleteResponse,
        )

    async def edit(
        self,
        custom_hostname_id: str,
        *,
        zone_id: str,
        custom_metadata: Dict[str, str] | Omit = omit,
        custom_origin_server: str | Omit = omit,
        custom_origin_sni: str | Omit = omit,
        ssl: custom_hostname_edit_params.SSL | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomHostnameEditResponse]:
        """Modify SSL configuration for a custom hostname.

        When sent with SSL config that
        matches existing config, used to indicate that hostname should pass domain
        control validation (DCV). Can also be used to change validation type, e.g., from
        'http' to 'email'. Bundle an existing certificate with another certificate by
        using the "custom_cert_bundle" field. The bundling process supports combining
        certificates as long as the following condition is met. One certificate must use
        the RSA algorithm, and the other must use the ECDSA algorithm.

        Args:
          zone_id: Identifier.

          custom_hostname_id: Identifier.

          custom_metadata: Unique key/value metadata for this hostname. These are per-hostname (customer)
              settings.

          custom_origin_server: a valid hostname that’s been added to your DNS zone as an A, AAAA, or CNAME
              record.

          custom_origin_sni: A hostname that will be sent to your custom origin server as SNI for TLS
              handshake. This can be a valid subdomain of the zone or custom origin server
              name or the string ':request_host_header:' which will cause the host header in
              the request to be used as SNI. Not configurable with default/fallback origin
              server.

          ssl: SSL properties used when creating the custom hostname.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_hostname_id:
            raise ValueError(f"Expected a non-empty value for `custom_hostname_id` but received {custom_hostname_id!r}")
        return await self._patch(
            path_template(
                "/zones/{zone_id}/custom_hostnames/{custom_hostname_id}",
                zone_id=zone_id,
                custom_hostname_id=custom_hostname_id,
            ),
            body=await async_maybe_transform(
                {
                    "custom_metadata": custom_metadata,
                    "custom_origin_server": custom_origin_server,
                    "custom_origin_sni": custom_origin_sni,
                    "ssl": ssl,
                },
                custom_hostname_edit_params.CustomHostnameEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomHostnameEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomHostnameEditResponse]], ResultWrapper[CustomHostnameEditResponse]),
        )

    async def get(
        self,
        custom_hostname_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomHostnameGetResponse]:
        """
        Retrieves detailed information about a specific custom hostname, including SSL
        certificate status, ownership verification, and origin configuration.

        Args:
          zone_id: Identifier.

          custom_hostname_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not custom_hostname_id:
            raise ValueError(f"Expected a non-empty value for `custom_hostname_id` but received {custom_hostname_id!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/custom_hostnames/{custom_hostname_id}",
                zone_id=zone_id,
                custom_hostname_id=custom_hostname_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomHostnameGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomHostnameGetResponse]], ResultWrapper[CustomHostnameGetResponse]),
        )


class CustomHostnamesResourceWithRawResponse:
    def __init__(self, custom_hostnames: CustomHostnamesResource) -> None:
        self._custom_hostnames = custom_hostnames

        self.create = to_raw_response_wrapper(
            custom_hostnames.create,
        )
        self.list = to_raw_response_wrapper(
            custom_hostnames.list,
        )
        self.delete = to_raw_response_wrapper(
            custom_hostnames.delete,
        )
        self.edit = to_raw_response_wrapper(
            custom_hostnames.edit,
        )
        self.get = to_raw_response_wrapper(
            custom_hostnames.get,
        )

    @cached_property
    def fallback_origin(self) -> FallbackOriginResourceWithRawResponse:
        return FallbackOriginResourceWithRawResponse(self._custom_hostnames.fallback_origin)

    @cached_property
    def certificate_pack(self) -> CertificatePackResourceWithRawResponse:
        return CertificatePackResourceWithRawResponse(self._custom_hostnames.certificate_pack)


class AsyncCustomHostnamesResourceWithRawResponse:
    def __init__(self, custom_hostnames: AsyncCustomHostnamesResource) -> None:
        self._custom_hostnames = custom_hostnames

        self.create = async_to_raw_response_wrapper(
            custom_hostnames.create,
        )
        self.list = async_to_raw_response_wrapper(
            custom_hostnames.list,
        )
        self.delete = async_to_raw_response_wrapper(
            custom_hostnames.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            custom_hostnames.edit,
        )
        self.get = async_to_raw_response_wrapper(
            custom_hostnames.get,
        )

    @cached_property
    def fallback_origin(self) -> AsyncFallbackOriginResourceWithRawResponse:
        return AsyncFallbackOriginResourceWithRawResponse(self._custom_hostnames.fallback_origin)

    @cached_property
    def certificate_pack(self) -> AsyncCertificatePackResourceWithRawResponse:
        return AsyncCertificatePackResourceWithRawResponse(self._custom_hostnames.certificate_pack)


class CustomHostnamesResourceWithStreamingResponse:
    def __init__(self, custom_hostnames: CustomHostnamesResource) -> None:
        self._custom_hostnames = custom_hostnames

        self.create = to_streamed_response_wrapper(
            custom_hostnames.create,
        )
        self.list = to_streamed_response_wrapper(
            custom_hostnames.list,
        )
        self.delete = to_streamed_response_wrapper(
            custom_hostnames.delete,
        )
        self.edit = to_streamed_response_wrapper(
            custom_hostnames.edit,
        )
        self.get = to_streamed_response_wrapper(
            custom_hostnames.get,
        )

    @cached_property
    def fallback_origin(self) -> FallbackOriginResourceWithStreamingResponse:
        return FallbackOriginResourceWithStreamingResponse(self._custom_hostnames.fallback_origin)

    @cached_property
    def certificate_pack(self) -> CertificatePackResourceWithStreamingResponse:
        return CertificatePackResourceWithStreamingResponse(self._custom_hostnames.certificate_pack)


class AsyncCustomHostnamesResourceWithStreamingResponse:
    def __init__(self, custom_hostnames: AsyncCustomHostnamesResource) -> None:
        self._custom_hostnames = custom_hostnames

        self.create = async_to_streamed_response_wrapper(
            custom_hostnames.create,
        )
        self.list = async_to_streamed_response_wrapper(
            custom_hostnames.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            custom_hostnames.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            custom_hostnames.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            custom_hostnames.get,
        )

    @cached_property
    def fallback_origin(self) -> AsyncFallbackOriginResourceWithStreamingResponse:
        return AsyncFallbackOriginResourceWithStreamingResponse(self._custom_hostnames.fallback_origin)

    @cached_property
    def certificate_pack(self) -> AsyncCertificatePackResourceWithStreamingResponse:
        return AsyncCertificatePackResourceWithStreamingResponse(self._custom_hostnames.certificate_pack)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.dex import colo_list_params
from ....types.zero_trust.dex.colo_list_response import ColoListResponse

__all__ = ["ColosResource", "AsyncColosResource"]


class ColosResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ColosResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ColosResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ColosResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ColosResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        from_: str,
        to: str,
        sort_by: Literal["fleet-status-usage", "application-tests-usage"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[ColoListResponse]:
        """
        List Cloudflare colos that account's devices were connected to during a time
        period, sorted by usage starting from the most used colo. Colos without traffic
        are also returned and sorted alphabetically.

        Args:
          from_: Start time for connection period in ISO (RFC3339 - ISO 8601) format

          to: End time for connection period in ISO (RFC3339 - ISO 8601) format

          sort_by: Type of usage that colos should be sorted by. If unspecified, returns all
              Cloudflare colos sorted alphabetically.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dex/colos", account_id=account_id),
            page=SyncSinglePage[ColoListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_": from_,
                        "to": to,
                        "sort_by": sort_by,
                    },
                    colo_list_params.ColoListParams,
                ),
            ),
            model=ColoListResponse,
        )


class AsyncColosResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncColosResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncColosResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncColosResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncColosResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        from_: str,
        to: str,
        sort_by: Literal["fleet-status-usage", "application-tests-usage"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ColoListResponse, AsyncSinglePage[ColoListResponse]]:
        """
        List Cloudflare colos that account's devices were connected to during a time
        period, sorted by usage starting from the most used colo. Colos without traffic
        are also returned and sorted alphabetically.

        Args:
          from_: Start time for connection period in ISO (RFC3339 - ISO 8601) format

          to: End time for connection period in ISO (RFC3339 - ISO 8601) format

          sort_by: Type of usage that colos should be sorted by. If unspecified, returns all
              Cloudflare colos sorted alphabetically.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dex/colos", account_id=account_id),
            page=AsyncSinglePage[ColoListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_": from_,
                        "to": to,
                        "sort_by": sort_by,
                    },
                    colo_list_params.ColoListParams,
                ),
            ),
            model=ColoListResponse,
        )


class ColosResourceWithRawResponse:
    def __init__(self, colos: ColosResource) -> None:
        self._colos = colos

        self.list = to_raw_response_wrapper(
            colos.list,
        )


class AsyncColosResourceWithRawResponse:
    def __init__(self, colos: AsyncColosResource) -> None:
        self._colos = colos

        self.list = async_to_raw_response_wrapper(
            colos.list,
        )


class ColosResourceWithStreamingResponse:
    def __init__(self, colos: ColosResource) -> None:
        self._colos = colos

        self.list = to_streamed_response_wrapper(
            colos.list,
        )


class AsyncColosResourceWithStreamingResponse:
    def __init__(self, colos: AsyncColosResource) -> None:
        self._colos = colos

        self.list = async_to_streamed_response_wrapper(
            colos.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["TailGetResponse"]


class TailGetResponse(BaseModel):
    id: str
    """Identifier."""

    expires_at: str

    url: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .schema import (
    SchemaResource,
    AsyncSchemaResource,
    SchemaResourceWithRawResponse,
    AsyncSchemaResourceWithRawResponse,
    SchemaResourceWithStreamingResponse,
    AsyncSchemaResourceWithStreamingResponse,
)

__all__ = [
    "SchemaResource",
    "AsyncSchemaResource",
    "SchemaResourceWithRawResponse",
    "AsyncSchemaResourceWithRawResponse",
    "SchemaResourceWithStreamingResponse",
    "AsyncSchemaResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ActiveSessionGetActiveSessionResponse", "Data"]


class Data(BaseModel):
    id: str
    """ID of the session"""

    associated_id: str
    """ID of the meeting this session is associated with.

    In the case of V2 meetings, it is always a UUID. In V1 meetings, it is a room
    name of the form `abcdef-ghijkl`
    """

    created_at: str
    """timestamp when session created"""

    live_participants: float
    """number of participants currently in the session"""

    max_concurrent_participants: float
    """number of maximum participants that were in the session"""

    meeting_display_name: str
    """Title of the meeting this session belongs to"""

    minutes_consumed: float
    """number of minutes consumed since the session started"""

    organization_id: str
    """App id that hosted this session"""

    started_at: str
    """timestamp when session started"""

    status: Literal["LIVE", "ENDED"]
    """current status of session"""

    type: Literal["meeting", "livestream", "participant"]
    """type of session"""

    updated_at: str
    """timestamp when session was last updated"""

    breakout_rooms: Optional[List[object]] = None

    ended_at: Optional[str] = None
    """timestamp when session ended"""

    meta: Optional[object] = None
    """Any meta data about session."""


class ActiveSessionGetActiveSessionResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DynamicRoutingCreateDeploymentParams"]


class DynamicRoutingCreateDeploymentParams(TypedDict, total=False):
    account_id: Required[str]

    gateway_id: Required[str]

    version_id: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["WebhookCreateWebhookParams"]


class WebhookCreateWebhookParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    events: Required[
        List[
            Literal[
                "meeting.started",
                "meeting.ended",
                "meeting.participantJoined",
                "meeting.participantLeft",
                "meeting.chatSynced",
                "recording.statusUpdate",
                "livestreaming.statusUpdate",
                "meeting.transcript",
                "meeting.summary",
            ]
        ]
    ]
    """Events that this webhook will get triggered by"""

    name: Required[str]
    """Name of the webhook"""

    url: Required[str]
    """URL this webhook will send events to"""

    enabled: bool
    """Set whether or not the webhook should be active when created"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["LivestreamCreateIndependentLivestreamParams"]


class LivestreamCreateIndependentLivestreamParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    name: Optional[str]
    """Name of the livestream"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.vulnerability_scanner.credential_sets import (
    credential_edit_params,
    credential_list_params,
    credential_create_params,
    credential_update_params,
)
from ....types.vulnerability_scanner.credential_sets.credential_get_response import CredentialGetResponse
from ....types.vulnerability_scanner.credential_sets.credential_edit_response import CredentialEditResponse
from ....types.vulnerability_scanner.credential_sets.credential_list_response import CredentialListResponse
from ....types.vulnerability_scanner.credential_sets.credential_create_response import CredentialCreateResponse
from ....types.vulnerability_scanner.credential_sets.credential_update_response import CredentialUpdateResponse

__all__ = ["CredentialsResource", "AsyncCredentialsResource"]


class CredentialsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CredentialsResourceWithStreamingResponse(self)

    def create(
        self,
        credential_set_id: str,
        *,
        account_id: str,
        location: Literal["header", "cookie"],
        location_name: str,
        name: str,
        value: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CredentialCreateResponse]:
        """
        Creates a new credential within a credential set.

        Args:
          account_id: Identifier.

          location: Where the credential is attached in outgoing requests.

          location_name: Name of the header or cookie where the credential is attached.

          name: Human-readable name.

          value: The credential value (e.g. API key, session token). Write-only. Never returned
              in responses.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials",
                account_id=account_id,
                credential_set_id=credential_set_id,
            ),
            body=maybe_transform(
                {
                    "location": location,
                    "location_name": location_name,
                    "name": name,
                    "value": value,
                },
                credential_create_params.CredentialCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CredentialCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CredentialCreateResponse]], ResultWrapper[CredentialCreateResponse]),
        )

    def update(
        self,
        credential_id: str,
        *,
        account_id: str,
        credential_set_id: str,
        location: Literal["header", "cookie"],
        location_name: str,
        name: str,
        value: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CredentialUpdateResponse]:
        """Replaces a credential.

        All fields must be provided.

        Args:
          account_id: Identifier.

          location: Where the credential is attached in outgoing requests.

          location_name: Name of the header or cookie where the credential is attached.

          name: Human-readable name.

          value: The credential value. Write-only. Never returned in responses.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        if not credential_id:
            raise ValueError(f"Expected a non-empty value for `credential_id` but received {credential_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials/{credential_id}",
                account_id=account_id,
                credential_set_id=credential_set_id,
                credential_id=credential_id,
            ),
            body=maybe_transform(
                {
                    "location": location,
                    "location_name": location_name,
                    "name": name,
                    "value": value,
                },
                credential_update_params.CredentialUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CredentialUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CredentialUpdateResponse]], ResultWrapper[CredentialUpdateResponse]),
        )

    def list(
        self,
        credential_set_id: str,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[CredentialListResponse]:
        """
        Returns all credentials within a credential set.

        Args:
          account_id: Identifier.

          page: Page number of paginated results.

          per_page: Number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials",
                account_id=account_id,
                credential_set_id=credential_set_id,
            ),
            page=SyncV4PagePaginationArray[CredentialListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    credential_list_params.CredentialListParams,
                ),
            ),
            model=CredentialListResponse,
        )

    def delete(
        self,
        credential_id: str,
        *,
        account_id: str,
        credential_set_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a credential.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        if not credential_id:
            raise ValueError(f"Expected a non-empty value for `credential_id` but received {credential_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials/{credential_id}",
                account_id=account_id,
                credential_set_id=credential_set_id,
                credential_id=credential_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def edit(
        self,
        credential_id: str,
        *,
        account_id: str,
        credential_set_id: str,
        location: Literal["header", "cookie"] | Omit = omit,
        location_name: str | Omit = omit,
        name: str | Omit = omit,
        value: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CredentialEditResponse]:
        """
        Updates a credential with only the provided fields; omitted fields remain
        unchanged.

        Args:
          account_id: Identifier.

          location: Where the credential is attached in outgoing requests.

          location_name: Name of the header or cookie where the credential is attached.

          name: Human-readable name.

          value: The credential value. Write-only. Never returned in responses.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        if not credential_id:
            raise ValueError(f"Expected a non-empty value for `credential_id` but received {credential_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials/{credential_id}",
                account_id=account_id,
                credential_set_id=credential_set_id,
                credential_id=credential_id,
            ),
            body=maybe_transform(
                {
                    "location": location,
                    "location_name": location_name,
                    "name": name,
                    "value": value,
                },
                credential_edit_params.CredentialEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CredentialEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CredentialEditResponse]], ResultWrapper[CredentialEditResponse]),
        )

    def get(
        self,
        credential_id: str,
        *,
        account_id: str,
        credential_set_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CredentialGetResponse]:
        """
        Returns a single credential by ID.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        if not credential_id:
            raise ValueError(f"Expected a non-empty value for `credential_id` but received {credential_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials/{credential_id}",
                account_id=account_id,
                credential_set_id=credential_set_id,
                credential_id=credential_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CredentialGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CredentialGetResponse]], ResultWrapper[CredentialGetResponse]),
        )


class AsyncCredentialsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCredentialsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCredentialsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCredentialsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCredentialsResourceWithStreamingResponse(self)

    async def create(
        self,
        credential_set_id: str,
        *,
        account_id: str,
        location: Literal["header", "cookie"],
        location_name: str,
        name: str,
        value: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CredentialCreateResponse]:
        """
        Creates a new credential within a credential set.

        Args:
          account_id: Identifier.

          location: Where the credential is attached in outgoing requests.

          location_name: Name of the header or cookie where the credential is attached.

          name: Human-readable name.

          value: The credential value (e.g. API key, session token). Write-only. Never returned
              in responses.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials",
                account_id=account_id,
                credential_set_id=credential_set_id,
            ),
            body=await async_maybe_transform(
                {
                    "location": location,
                    "location_name": location_name,
                    "name": name,
                    "value": value,
                },
                credential_create_params.CredentialCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CredentialCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CredentialCreateResponse]], ResultWrapper[CredentialCreateResponse]),
        )

    async def update(
        self,
        credential_id: str,
        *,
        account_id: str,
        credential_set_id: str,
        location: Literal["header", "cookie"],
        location_name: str,
        name: str,
        value: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CredentialUpdateResponse]:
        """Replaces a credential.

        All fields must be provided.

        Args:
          account_id: Identifier.

          location: Where the credential is attached in outgoing requests.

          location_name: Name of the header or cookie where the credential is attached.

          name: Human-readable name.

          value: The credential value. Write-only. Never returned in responses.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        if not credential_id:
            raise ValueError(f"Expected a non-empty value for `credential_id` but received {credential_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials/{credential_id}",
                account_id=account_id,
                credential_set_id=credential_set_id,
                credential_id=credential_id,
            ),
            body=await async_maybe_transform(
                {
                    "location": location,
                    "location_name": location_name,
                    "name": name,
                    "value": value,
                },
                credential_update_params.CredentialUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CredentialUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CredentialUpdateResponse]], ResultWrapper[CredentialUpdateResponse]),
        )

    def list(
        self,
        credential_set_id: str,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CredentialListResponse, AsyncV4PagePaginationArray[CredentialListResponse]]:
        """
        Returns all credentials within a credential set.

        Args:
          account_id: Identifier.

          page: Page number of paginated results.

          per_page: Number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials",
                account_id=account_id,
                credential_set_id=credential_set_id,
            ),
            page=AsyncV4PagePaginationArray[CredentialListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    credential_list_params.CredentialListParams,
                ),
            ),
            model=CredentialListResponse,
        )

    async def delete(
        self,
        credential_id: str,
        *,
        account_id: str,
        credential_set_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a credential.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        if not credential_id:
            raise ValueError(f"Expected a non-empty value for `credential_id` but received {credential_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials/{credential_id}",
                account_id=account_id,
                credential_set_id=credential_set_id,
                credential_id=credential_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def edit(
        self,
        credential_id: str,
        *,
        account_id: str,
        credential_set_id: str,
        location: Literal["header", "cookie"] | Omit = omit,
        location_name: str | Omit = omit,
        name: str | Omit = omit,
        value: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CredentialEditResponse]:
        """
        Updates a credential with only the provided fields; omitted fields remain
        unchanged.

        Args:
          account_id: Identifier.

          location: Where the credential is attached in outgoing requests.

          location_name: Name of the header or cookie where the credential is attached.

          name: Human-readable name.

          value: The credential value. Write-only. Never returned in responses.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        if not credential_id:
            raise ValueError(f"Expected a non-empty value for `credential_id` but received {credential_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials/{credential_id}",
                account_id=account_id,
                credential_set_id=credential_set_id,
                credential_id=credential_id,
            ),
            body=await async_maybe_transform(
                {
                    "location": location,
                    "location_name": location_name,
                    "name": name,
                    "value": value,
                },
                credential_edit_params.CredentialEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CredentialEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CredentialEditResponse]], ResultWrapper[CredentialEditResponse]),
        )

    async def get(
        self,
        credential_id: str,
        *,
        account_id: str,
        credential_set_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CredentialGetResponse]:
        """
        Returns a single credential by ID.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not credential_set_id:
            raise ValueError(f"Expected a non-empty value for `credential_set_id` but received {credential_set_id!r}")
        if not credential_id:
            raise ValueError(f"Expected a non-empty value for `credential_id` but received {credential_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/vuln_scanner/credential_sets/{credential_set_id}/credentials/{credential_id}",
                account_id=account_id,
                credential_set_id=credential_set_id,
                credential_id=credential_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CredentialGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CredentialGetResponse]], ResultWrapper[CredentialGetResponse]),
        )


class CredentialsResourceWithRawResponse:
    def __init__(self, credentials: CredentialsResource) -> None:
        self._credentials = credentials

        self.create = to_raw_response_wrapper(
            credentials.create,
        )
        self.update = to_raw_response_wrapper(
            credentials.update,
        )
        self.list = to_raw_response_wrapper(
            credentials.list,
        )
        self.delete = to_raw_response_wrapper(
            credentials.delete,
        )
        self.edit = to_raw_response_wrapper(
            credentials.edit,
        )
        self.get = to_raw_response_wrapper(
            credentials.get,
        )


class AsyncCredentialsResourceWithRawResponse:
    def __init__(self, credentials: AsyncCredentialsResource) -> None:
        self._credentials = credentials

        self.create = async_to_raw_response_wrapper(
            credentials.create,
        )
        self.update = async_to_raw_response_wrapper(
            credentials.update,
        )
        self.list = async_to_raw_response_wrapper(
            credentials.list,
        )
        self.delete = async_to_raw_response_wrapper(
            credentials.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            credentials.edit,
        )
        self.get = async_to_raw_response_wrapper(
            credentials.get,
        )


class CredentialsResourceWithStreamingResponse:
    def __init__(self, credentials: CredentialsResource) -> None:
        self._credentials = credentials

        self.create = to_streamed_response_wrapper(
            credentials.create,
        )
        self.update = to_streamed_response_wrapper(
            credentials.update,
        )
        self.list = to_streamed_response_wrapper(
            credentials.list,
        )
        self.delete = to_streamed_response_wrapper(
            credentials.delete,
        )
        self.edit = to_streamed_response_wrapper(
            credentials.edit,
        )
        self.get = to_streamed_response_wrapper(
            credentials.get,
        )


class AsyncCredentialsResourceWithStreamingResponse:
    def __init__(self, credentials: AsyncCredentialsResource) -> None:
        self._credentials = credentials

        self.create = async_to_streamed_response_wrapper(
            credentials.create,
        )
        self.update = async_to_streamed_response_wrapper(
            credentials.update,
        )
        self.list = async_to_streamed_response_wrapper(
            credentials.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            credentials.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            credentials.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            credentials.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["WebhookEditWebhookParams"]


class WebhookEditWebhookParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    app_id: Required[str]
    """The app identifier tag."""

    enabled: bool

    events: List[
        Literal[
            "meeting.started",
            "meeting.ended",
            "meeting.participantJoined",
            "meeting.participantLeft",
            "recording.statusUpdate",
            "livestreaming.statusUpdate",
            "meeting.chatSynced",
            "meeting.transcript",
            "meeting.summary",
        ]
    ]
    """Events that the webhook will get triggered by"""

    name: str
    """Name of the webhook"""

    url: str
    """URL the webhook will send events to"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .topup import (
    TopupResource,
    AsyncTopupResource,
    TopupResourceWithRawResponse,
    AsyncTopupResourceWithRawResponse,
    TopupResourceWithStreamingResponse,
    AsyncTopupResourceWithStreamingResponse,
)
from .billing import (
    BillingResource,
    AsyncBillingResource,
    BillingResourceWithRawResponse,
    AsyncBillingResourceWithRawResponse,
    BillingResourceWithStreamingResponse,
    AsyncBillingResourceWithStreamingResponse,
)
from .spending_limit import (
    SpendingLimitResource,
    AsyncSpendingLimitResource,
    SpendingLimitResourceWithRawResponse,
    AsyncSpendingLimitResourceWithRawResponse,
    SpendingLimitResourceWithStreamingResponse,
    AsyncSpendingLimitResourceWithStreamingResponse,
)

__all__ = [
    "TopupResource",
    "AsyncTopupResource",
    "TopupResourceWithRawResponse",
    "AsyncTopupResourceWithRawResponse",
    "TopupResourceWithStreamingResponse",
    "AsyncTopupResourceWithStreamingResponse",
    "SpendingLimitResource",
    "AsyncSpendingLimitResource",
    "SpendingLimitResourceWithRawResponse",
    "AsyncSpendingLimitResourceWithRawResponse",
    "SpendingLimitResourceWithStreamingResponse",
    "AsyncSpendingLimitResourceWithStreamingResponse",
    "BillingResource",
    "AsyncBillingResource",
    "BillingResourceWithRawResponse",
    "AsyncBillingResourceWithRawResponse",
    "BillingResourceWithStreamingResponse",
    "AsyncBillingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["LivestreamStopLivestreamingAMeetingResponse", "Data"]


class Data(BaseModel):
    message: Optional[str] = None


class LivestreamStopLivestreamingAMeetingResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ProfileListParams"]


class ProfileListParams(TypedDict, total=False):
    account_id: Required[str]

    all: bool
    """
    Return all profiles, including those that current account does not have access
    to.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .apps import (
    AppsResource,
    AsyncAppsResource,
    AppsResourceWithRawResponse,
    AsyncAppsResourceWithRawResponse,
    AppsResourceWithStreamingResponse,
    AsyncAppsResourceWithStreamingResponse,
)
from .presets import (
    PresetsResource,
    AsyncPresetsResource,
    PresetsResourceWithRawResponse,
    AsyncPresetsResourceWithRawResponse,
    PresetsResourceWithStreamingResponse,
    AsyncPresetsResourceWithStreamingResponse,
)
from .meetings import (
    MeetingsResource,
    AsyncMeetingsResource,
    MeetingsResourceWithRawResponse,
    AsyncMeetingsResourceWithRawResponse,
    MeetingsResourceWithStreamingResponse,
    AsyncMeetingsResourceWithStreamingResponse,
)
from .sessions import (
    SessionsResource,
    AsyncSessionsResource,
    SessionsResourceWithRawResponse,
    AsyncSessionsResourceWithRawResponse,
    SessionsResourceWithStreamingResponse,
    AsyncSessionsResourceWithStreamingResponse,
)
from .webhooks import (
    WebhooksResource,
    AsyncWebhooksResource,
    WebhooksResourceWithRawResponse,
    AsyncWebhooksResourceWithRawResponse,
    WebhooksResourceWithStreamingResponse,
    AsyncWebhooksResourceWithStreamingResponse,
)
from .analytics import (
    AnalyticsResource,
    AsyncAnalyticsResource,
    AnalyticsResourceWithRawResponse,
    AsyncAnalyticsResourceWithRawResponse,
    AnalyticsResourceWithStreamingResponse,
    AsyncAnalyticsResourceWithStreamingResponse,
)
from .recordings import (
    RecordingsResource,
    AsyncRecordingsResource,
    RecordingsResourceWithRawResponse,
    AsyncRecordingsResourceWithRawResponse,
    RecordingsResourceWithStreamingResponse,
    AsyncRecordingsResourceWithStreamingResponse,
)
from .livestreams import (
    LivestreamsResource,
    AsyncLivestreamsResource,
    LivestreamsResourceWithRawResponse,
    AsyncLivestreamsResourceWithRawResponse,
    LivestreamsResourceWithStreamingResponse,
    AsyncLivestreamsResourceWithStreamingResponse,
)
from .realtime_kit import (
    RealtimeKitResource,
    AsyncRealtimeKitResource,
    RealtimeKitResourceWithRawResponse,
    AsyncRealtimeKitResourceWithRawResponse,
    RealtimeKitResourceWithStreamingResponse,
    AsyncRealtimeKitResourceWithStreamingResponse,
)
from .active_session import (
    ActiveSessionResource,
    AsyncActiveSessionResource,
    ActiveSessionResourceWithRawResponse,
    AsyncActiveSessionResourceWithRawResponse,
    ActiveSessionResourceWithStreamingResponse,
    AsyncActiveSessionResourceWithStreamingResponse,
)

__all__ = [
    "AppsResource",
    "AsyncAppsResource",
    "AppsResourceWithRawResponse",
    "AsyncAppsResourceWithRawResponse",
    "AppsResourceWithStreamingResponse",
    "AsyncAppsResourceWithStreamingResponse",
    "MeetingsResource",
    "AsyncMeetingsResource",
    "MeetingsResourceWithRawResponse",
    "AsyncMeetingsResourceWithRawResponse",
    "MeetingsResourceWithStreamingResponse",
    "AsyncMeetingsResourceWithStreamingResponse",
    "PresetsResource",
    "AsyncPresetsResource",
    "PresetsResourceWithRawResponse",
    "AsyncPresetsResourceWithRawResponse",
    "PresetsResourceWithStreamingResponse",
    "AsyncPresetsResourceWithStreamingResponse",
    "SessionsResource",
    "AsyncSessionsResource",
    "SessionsResourceWithRawResponse",
    "AsyncSessionsResourceWithRawResponse",
    "SessionsResourceWithStreamingResponse",
    "AsyncSessionsResourceWithStreamingResponse",
    "RecordingsResource",
    "AsyncRecordingsResource",
    "RecordingsResourceWithRawResponse",
    "AsyncRecordingsResourceWithRawResponse",
    "RecordingsResourceWithStreamingResponse",
    "AsyncRecordingsResourceWithStreamingResponse",
    "WebhooksResource",
    "AsyncWebhooksResource",
    "WebhooksResourceWithRawResponse",
    "AsyncWebhooksResourceWithRawResponse",
    "WebhooksResourceWithStreamingResponse",
    "AsyncWebhooksResourceWithStreamingResponse",
    "ActiveSessionResource",
    "AsyncActiveSessionResource",
    "ActiveSessionResourceWithRawResponse",
    "AsyncActiveSessionResourceWithRawResponse",
    "ActiveSessionResourceWithStreamingResponse",
    "AsyncActiveSessionResourceWithStreamingResponse",
    "LivestreamsResource",
    "AsyncLivestreamsResource",
    "LivestreamsResourceWithRawResponse",
    "AsyncLivestreamsResourceWithRawResponse",
    "LivestreamsResourceWithStreamingResponse",
    "AsyncLivestreamsResourceWithStreamingResponse",
    "AnalyticsResource",
    "AsyncAnalyticsResource",
    "AnalyticsResourceWithRawResponse",
    "AsyncAnalyticsResourceWithRawResponse",
    "AnalyticsResourceWithStreamingResponse",
    "AsyncAnalyticsResourceWithStreamingResponse",
    "RealtimeKitResource",
    "AsyncRealtimeKitResource",
    "RealtimeKitResourceWithRawResponse",
    "AsyncRealtimeKitResourceWithRawResponse",
    "RealtimeKitResourceWithStreamingResponse",
    "AsyncRealtimeKitResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.devices.resilience import global_warp_override_create_params
from .....types.zero_trust.devices.resilience.global_warp_override_get_response import GlobalWARPOverrideGetResponse
from .....types.zero_trust.devices.resilience.global_warp_override_create_response import (
    GlobalWARPOverrideCreateResponse,
)

__all__ = ["GlobalWARPOverrideResource", "AsyncGlobalWARPOverrideResource"]


class GlobalWARPOverrideResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> GlobalWARPOverrideResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return GlobalWARPOverrideResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GlobalWARPOverrideResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return GlobalWARPOverrideResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        disconnect: bool,
        justification: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GlobalWARPOverrideCreateResponse]:
        """
        Sets the Global WARP override state.

        Args:
          disconnect: Disconnects all devices on the account using Global WARP override.

          justification: Reasoning for setting the Global WARP override state. This will be surfaced in
              the audit log.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/devices/resilience/disconnect", account_id=account_id),
            body=maybe_transform(
                {
                    "disconnect": disconnect,
                    "justification": justification,
                },
                global_warp_override_create_params.GlobalWARPOverrideCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GlobalWARPOverrideCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[GlobalWARPOverrideCreateResponse]], ResultWrapper[GlobalWARPOverrideCreateResponse]
            ),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GlobalWARPOverrideGetResponse]:
        """
        Fetch the Global WARP override state.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/devices/resilience/disconnect", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GlobalWARPOverrideGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GlobalWARPOverrideGetResponse]], ResultWrapper[GlobalWARPOverrideGetResponse]),
        )


class AsyncGlobalWARPOverrideResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncGlobalWARPOverrideResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGlobalWARPOverrideResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGlobalWARPOverrideResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncGlobalWARPOverrideResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        disconnect: bool,
        justification: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GlobalWARPOverrideCreateResponse]:
        """
        Sets the Global WARP override state.

        Args:
          disconnect: Disconnects all devices on the account using Global WARP override.

          justification: Reasoning for setting the Global WARP override state. This will be surfaced in
              the audit log.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/devices/resilience/disconnect", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "disconnect": disconnect,
                    "justification": justification,
                },
                global_warp_override_create_params.GlobalWARPOverrideCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GlobalWARPOverrideCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[GlobalWARPOverrideCreateResponse]], ResultWrapper[GlobalWARPOverrideCreateResponse]
            ),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GlobalWARPOverrideGetResponse]:
        """
        Fetch the Global WARP override state.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/devices/resilience/disconnect", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GlobalWARPOverrideGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GlobalWARPOverrideGetResponse]], ResultWrapper[GlobalWARPOverrideGetResponse]),
        )


class GlobalWARPOverrideResourceWithRawResponse:
    def __init__(self, global_warp_override: GlobalWARPOverrideResource) -> None:
        self._global_warp_override = global_warp_override

        self.create = to_raw_response_wrapper(
            global_warp_override.create,
        )
        self.get = to_raw_response_wrapper(
            global_warp_override.get,
        )


class AsyncGlobalWARPOverrideResourceWithRawResponse:
    def __init__(self, global_warp_override: AsyncGlobalWARPOverrideResource) -> None:
        self._global_warp_override = global_warp_override

        self.create = async_to_raw_response_wrapper(
            global_warp_override.create,
        )
        self.get = async_to_raw_response_wrapper(
            global_warp_override.get,
        )


class GlobalWARPOverrideResourceWithStreamingResponse:
    def __init__(self, global_warp_override: GlobalWARPOverrideResource) -> None:
        self._global_warp_override = global_warp_override

        self.create = to_streamed_response_wrapper(
            global_warp_override.create,
        )
        self.get = to_streamed_response_wrapper(
            global_warp_override.get,
        )


class AsyncGlobalWARPOverrideResourceWithStreamingResponse:
    def __init__(self, global_warp_override: AsyncGlobalWARPOverrideResource) -> None:
        self._global_warp_override = global_warp_override

        self.create = async_to_streamed_response_wrapper(
            global_warp_override.create,
        )
        self.get = async_to_streamed_response_wrapper(
            global_warp_override.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "TrafficAnomalyGetResponse",
    "TrafficAnomaly",
    "TrafficAnomalyASNDetails",
    "TrafficAnomalyASNDetailsLocations",
    "TrafficAnomalyLocationDetails",
    "TrafficAnomalyOriginDetails",
]


class TrafficAnomalyASNDetailsLocations(BaseModel):
    code: str

    name: str


class TrafficAnomalyASNDetails(BaseModel):
    asn: str

    name: str

    locations: Optional[TrafficAnomalyASNDetailsLocations] = None


class TrafficAnomalyLocationDetails(BaseModel):
    code: str

    name: str


class TrafficAnomalyOriginDetails(BaseModel):
    name: str

    origin: str


class TrafficAnomaly(BaseModel):
    start_date: str = FieldInfo(alias="startDate")

    status: str

    type: str

    uuid: str

    asn_details: Optional[TrafficAnomalyASNDetails] = FieldInfo(alias="asnDetails", default=None)

    end_date: Optional[datetime] = FieldInfo(alias="endDate", default=None)

    location_details: Optional[TrafficAnomalyLocationDetails] = FieldInfo(alias="locationDetails", default=None)

    origin_details: Optional[TrafficAnomalyOriginDetails] = FieldInfo(alias="originDetails", default=None)

    visible_in_data_sources: Optional[List[str]] = FieldInfo(alias="visibleInDataSources", default=None)


class TrafficAnomalyGetResponse(BaseModel):
    traffic_anomalies: List[TrafficAnomaly] = FieldInfo(alias="trafficAnomalies")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["EdgeCreateParams"]


class EdgeCreateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    fields: str
    """Comma-separated list of fields."""

    filter: str
    """Filters to drill down into specific events."""

    sample: int
    """
    The sample parameter is the sample rate of the records set by the client:
    "sample": 1 is 100% of records "sample": 10 is 10% and so on.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ....._types import FileTypes, SequenceNotStr
from ....workers.migration_step_param import MigrationStepParam
from ....workers.single_step_migration_param import SingleStepMigrationParam
from ....workers.scripts.consumer_script_param import ConsumerScriptParam

__all__ = [
    "ScriptUpdateParams",
    "Metadata",
    "MetadataAssets",
    "MetadataAssetsConfig",
    "MetadataBinding",
    "MetadataBindingWorkersBindingKindAI",
    "MetadataBindingWorkersBindingKindAISearch",
    "MetadataBindingWorkersBindingKindAISearchNamespace",
    "MetadataBindingWorkersBindingKindAnalyticsEngine",
    "MetadataBindingWorkersBindingKindAssets",
    "MetadataBindingWorkersBindingKindBrowser",
    "MetadataBindingWorkersBindingKindD1",
    "MetadataBindingWorkersBindingKindDataBlob",
    "MetadataBindingWorkersBindingKindDispatchNamespace",
    "MetadataBindingWorkersBindingKindDispatchNamespaceOutbound",
    "MetadataBindingWorkersBindingKindDispatchNamespaceOutboundParam",
    "MetadataBindingWorkersBindingKindDispatchNamespaceOutboundWorker",
    "MetadataBindingWorkersBindingKindDurableObjectNamespace",
    "MetadataBindingWorkersBindingKindHyperdrive",
    "MetadataBindingWorkersBindingKindInherit",
    "MetadataBindingWorkersBindingKindImages",
    "MetadataBindingWorkersBindingKindJson",
    "MetadataBindingWorkersBindingKindKVNamespace",
    "MetadataBindingWorkersBindingKindMedia",
    "MetadataBindingWorkersBindingKindMTLSCertificate",
    "MetadataBindingWorkersBindingKindPlainText",
    "MetadataBindingWorkersBindingKindPipelines",
    "MetadataBindingWorkersBindingKindQueue",
    "MetadataBindingWorkersBindingKindRatelimit",
    "MetadataBindingWorkersBindingKindRatelimitSimple",
    "MetadataBindingWorkersBindingKindR2Bucket",
    "MetadataBindingWorkersBindingKindSecretText",
    "MetadataBindingWorkersBindingKindSendEmail",
    "MetadataBindingWorkersBindingKindService",
    "MetadataBindingWorkersBindingKindTextBlob",
    "MetadataBindingWorkersBindingKindVectorize",
    "MetadataBindingWorkersBindingKindVersionMetadata",
    "MetadataBindingWorkersBindingKindSecretsStoreSecret",
    "MetadataBindingWorkersBindingKindFlagship",
    "MetadataBindingWorkersBindingKindSecretKey",
    "MetadataBindingWorkersBindingKindWorkflow",
    "MetadataBindingWorkersBindingKindWasmModule",
    "MetadataBindingWorkersBindingKindVPCService",
    "MetadataBindingWorkersBindingKindVPCNetwork",
    "MetadataLimits",
    "MetadataMigrations",
    "MetadataMigrationsWorkersMultipleStepMigrations",
    "MetadataObservability",
    "MetadataObservabilityLogs",
    "MetadataObservabilityTraces",
    "MetadataPlacement",
    "MetadataPlacementUnionMember0",
    "MetadataPlacementUnionMember1",
    "MetadataPlacementUnionMember2",
    "MetadataPlacementUnionMember3",
    "MetadataPlacementUnionMember4",
    "MetadataPlacementUnionMember5",
    "MetadataPlacementUnionMember6",
    "MetadataPlacementUnionMember7",
    "MetadataPlacementUnionMember7Target",
    "MetadataPlacementUnionMember7TargetRegion",
    "MetadataPlacementUnionMember7TargetHostname",
    "MetadataPlacementUnionMember7TargetHost",
]


class ScriptUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    dispatch_namespace: Required[str]
    """Name of the Workers for Platforms dispatch namespace."""

    metadata: Required[Metadata]
    """JSON-encoded metadata about the uploaded parts and Worker configuration."""

    bindings_inherit: Literal["strict"]
    """
    When set to "strict", the upload will fail if any `inherit` type bindings cannot
    be resolved against the previous version of the script. Without this,
    unresolvable inherit bindings are silently dropped.
    """

    files: SequenceNotStr[FileTypes]
    """An array of modules (often JavaScript files) comprising a Worker script.

    At least one module must be present and referenced in the metadata as
    `main_module` or `body_part` by filename.<br/>Possible Content-Type(s) are:
    `application/javascript+module`, `text/javascript+module`,
    `application/javascript`, `text/javascript`, `text/x-python`,
    `text/x-python-requirement`, `application/wasm`, `text/plain`,
    `application/octet-stream`, `application/source-map`.
    """


class MetadataAssetsConfig(TypedDict, total=False):
    """Configuration for assets within a Worker."""

    _headers: str
    """
    The contents of a \\__headers file (used to attach custom headers on asset
    responses).
    """

    _redirects: str
    """
    The contents of a \\__redirects file (used to apply redirects or proxy paths ahead
    of asset serving).
    """

    html_handling: Literal["auto-trailing-slash", "force-trailing-slash", "drop-trailing-slash", "none"]
    """Determines the redirects and rewrites of requests for HTML content."""

    not_found_handling: Literal["none", "404-page", "single-page-application"]
    """
    Determines the response when a request does not match a static asset, and there
    is no Worker script.
    """

    run_worker_first: Union[SequenceNotStr[str], bool]
    """Contains a list path rules to control routing to either the Worker or assets.

    Glob (\\**) and negative (!) rules are supported. Rules must start with either '/'
    or '!/'. At least one non-negative rule must be provided, and negative rules
    have higher precedence than non-negative rules.
    """

    serve_directly: bool
    """
    When true and the incoming request matches an asset, that will be served instead
    of invoking the Worker script. When false, requests will always invoke the
    Worker script.
    """


class MetadataAssets(TypedDict, total=False):
    """Configuration for assets within a Worker."""

    config: MetadataAssetsConfig
    """Configuration for assets within a Worker."""

    jwt: str
    """Token provided upon successful upload of all files from a registered manifest."""


class MetadataBindingWorkersBindingKindAI(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["ai"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindAISearch(TypedDict, total=False):
    instance_name: Required[str]
    """The user-chosen instance name.

    Must exist at deploy time. The worker can search, chat, update, and manage
    items/jobs on this instance.
    """

    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["ai_search"]]
    """The kind of resource that the binding provides."""

    namespace: str
    """The namespace the instance belongs to.

    Defaults to "default" if omitted. Customers who don't use namespaces can simply
    omit this field.
    """


class MetadataBindingWorkersBindingKindAISearchNamespace(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    namespace: Required[str]
    """The user-chosen namespace name.

    Must exist before deploy -- Wrangler handles auto-creation on deploy failure (R2
    bucket pattern). The "default" namespace is auto-created by config-api for new
    accounts. Grants full access (CRUD + search + chat) to all instances within the
    namespace.
    """

    type: Required[Literal["ai_search_namespace"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindAnalyticsEngine(TypedDict, total=False):
    dataset: Required[str]
    """The name of the dataset to bind to."""

    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["analytics_engine"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindAssets(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["assets"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindBrowser(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["browser"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindD1(TypedDict, total=False):
    database_id: Required[str]
    """Identifier of the D1 database to bind to."""

    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["d1"]]
    """The kind of resource that the binding provides."""

    id: str
    """Identifier of the D1 database to bind to."""


class MetadataBindingWorkersBindingKindDataBlob(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    part: Required[str]
    """The name of the file containing the data content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Required[Literal["data_blob"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindDispatchNamespaceOutboundParam(TypedDict, total=False):
    name: Required[str]
    """Name of the parameter."""


class MetadataBindingWorkersBindingKindDispatchNamespaceOutboundWorker(TypedDict, total=False):
    """Outbound worker."""

    entrypoint: str
    """Entrypoint to invoke on the outbound worker."""

    environment: str
    """Environment of the outbound worker."""

    service: str
    """Name of the outbound worker."""


class MetadataBindingWorkersBindingKindDispatchNamespaceOutbound(TypedDict, total=False):
    """Outbound worker."""

    params: Iterable[MetadataBindingWorkersBindingKindDispatchNamespaceOutboundParam]
    """
    Pass information from the Dispatch Worker to the Outbound Worker through the
    parameters.
    """

    worker: MetadataBindingWorkersBindingKindDispatchNamespaceOutboundWorker
    """Outbound worker."""


class MetadataBindingWorkersBindingKindDispatchNamespace(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    namespace: Required[str]
    """The name of the dispatch namespace."""

    type: Required[Literal["dispatch_namespace"]]
    """The kind of resource that the binding provides."""

    outbound: MetadataBindingWorkersBindingKindDispatchNamespaceOutbound
    """Outbound worker."""


class MetadataBindingWorkersBindingKindDurableObjectNamespace(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["durable_object_namespace"]]
    """The kind of resource that the binding provides."""

    class_name: str
    """The exported class name of the Durable Object."""

    dispatch_namespace: str
    """The dispatch namespace the Durable Object script belongs to."""

    environment: str
    """The environment of the script_name to bind to."""

    namespace_id: str
    """Namespace identifier tag."""

    script_name: str
    """
    The script where the Durable Object is defined, if it is external to this
    Worker.
    """


class MetadataBindingWorkersBindingKindHyperdrive(TypedDict, total=False):
    id: Required[str]
    """Identifier of the Hyperdrive connection to bind to."""

    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["hyperdrive"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindInherit(TypedDict, total=False):
    name: Required[str]
    """The name of the inherited binding."""

    type: Required[Literal["inherit"]]
    """The kind of resource that the binding provides."""

    old_name: str
    """The old name of the inherited binding.

    If set, the binding will be renamed from `old_name` to `name` in the new
    version. If not set, the binding will keep the same name between versions.
    """

    version_id: str
    """
    Identifier for the version to inherit the binding from, which can be the version
    ID or the literal "latest" to inherit from the latest version. Defaults to
    inheriting the binding from the latest version.
    """


class MetadataBindingWorkersBindingKindImages(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["images"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindJson(TypedDict, total=False):
    json: Required[object]
    """JSON data to use."""

    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["json"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindKVNamespace(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    namespace_id: Required[str]
    """Namespace identifier tag."""

    type: Required[Literal["kv_namespace"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindMedia(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["media"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindMTLSCertificate(TypedDict, total=False):
    certificate_id: Required[str]
    """Identifier of the certificate to bind to."""

    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["mtls_certificate"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindPlainText(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    text: Required[str]
    """The text value to use."""

    type: Required[Literal["plain_text"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindPipelines(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    pipeline: Required[str]
    """Name of the Pipeline to bind to."""

    type: Required[Literal["pipelines"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindQueue(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    queue_name: Required[str]
    """Name of the Queue to bind to."""

    type: Required[Literal["queue"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindRatelimitSimple(TypedDict, total=False):
    """The rate limit configuration."""

    limit: Required[float]
    """The limit (requests per period)."""

    period: Required[int]
    """The period in seconds."""

    mitigation_timeout: int
    """
    Duration in seconds to apply the mitigation action after the rate limit is
    exceeded. Valid values are 0 (disabled), 10, or multiples of 60 up to 86400.
    Must be greater than or equal to the period when non-zero.
    """


class MetadataBindingWorkersBindingKindRatelimit(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    namespace_id: Required[str]
    """Identifier of the rate limit namespace to bind to."""

    simple: Required[MetadataBindingWorkersBindingKindRatelimitSimple]
    """The rate limit configuration."""

    type: Required[Literal["ratelimit"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindR2Bucket(TypedDict, total=False):
    bucket_name: Required[str]
    """R2 bucket to bind to."""

    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["r2_bucket"]]
    """The kind of resource that the binding provides."""

    jurisdiction: Literal["eu", "fedramp", "fedramp-high"]
    """
    The
    [jurisdiction](https://developers.cloudflare.com/r2/reference/data-location/#jurisdictional-restrictions)
    of the R2 bucket.
    """


class MetadataBindingWorkersBindingKindSecretText(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    text: Required[str]
    """The secret value to use."""

    type: Required[Literal["secret_text"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindSendEmail(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["send_email"]]
    """The kind of resource that the binding provides."""

    allowed_destination_addresses: SequenceNotStr[str]
    """List of allowed destination addresses."""

    allowed_sender_addresses: SequenceNotStr[str]
    """List of allowed sender addresses."""

    destination_address: str
    """Destination address for the email."""


class MetadataBindingWorkersBindingKindService(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    service: Required[str]
    """Name of Worker to bind to."""

    type: Required[Literal["service"]]
    """The kind of resource that the binding provides."""

    entrypoint: str
    """Entrypoint to invoke on the target Worker."""

    environment: str
    """Optional environment if the Worker utilizes one."""


class MetadataBindingWorkersBindingKindTextBlob(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    part: Required[str]
    """The name of the file containing the text content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Required[Literal["text_blob"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindVectorize(TypedDict, total=False):
    index_name: Required[str]
    """Name of the Vectorize index to bind to."""

    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["vectorize"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindVersionMetadata(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["version_metadata"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindSecretsStoreSecret(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    secret_name: Required[str]
    """Name of the secret in the store."""

    store_id: Required[str]
    """ID of the store containing the secret."""

    type: Required[Literal["secrets_store_secret"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindFlagship(TypedDict, total=False):
    app_id: Required[str]
    """ID of the Flagship app to bind to for feature flag evaluation."""

    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["flagship"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindSecretKey(TypedDict, total=False):
    algorithm: Required[object]
    """Algorithm-specific key parameters.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#algorithm).
    """

    format: Required[Literal["raw", "pkcs8", "spki", "jwk"]]
    """Data format of the key.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#format).
    """

    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["secret_key"]]
    """The kind of resource that the binding provides."""

    usages: Required[
        List[Literal["encrypt", "decrypt", "sign", "verify", "deriveKey", "deriveBits", "wrapKey", "unwrapKey"]]
    ]
    """Allowed operations with the key.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#keyUsages).
    """

    key_base64: str
    """Base64-encoded key data. Required if `format` is "raw", "pkcs8", or "spki"."""

    key_jwk: object
    """
    Key data in
    [JSON Web Key](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#json_web_key)
    format. Required if `format` is "jwk".
    """


class MetadataBindingWorkersBindingKindWorkflow(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["workflow"]]
    """The kind of resource that the binding provides."""

    workflow_name: Required[str]
    """Name of the Workflow to bind to."""

    class_name: str
    """Class name of the Workflow.

    Should only be provided if the Workflow belongs to this script.
    """

    script_name: str
    """Script name that contains the Workflow.

    If not provided, defaults to this script name.
    """


class MetadataBindingWorkersBindingKindWasmModule(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    part: Required[str]
    """The name of the file containing the WebAssembly module content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Required[Literal["wasm_module"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindVPCService(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    service_id: Required[str]
    """Identifier of the VPC service to bind to."""

    type: Required[Literal["vpc_service"]]
    """The kind of resource that the binding provides."""


class MetadataBindingWorkersBindingKindVPCNetwork(TypedDict, total=False):
    name: Required[str]
    """A JavaScript variable name for the binding."""

    type: Required[Literal["vpc_network"]]
    """The kind of resource that the binding provides."""

    network_id: str
    """Identifier of the network to bind to.

    Only "cf1:network" is currently supported. Mutually exclusive with tunnel_id.
    """

    tunnel_id: str
    """UUID of the Cloudflare Tunnel to bind to. Mutually exclusive with network_id."""


MetadataBinding: TypeAlias = Union[
    MetadataBindingWorkersBindingKindAI,
    MetadataBindingWorkersBindingKindAISearch,
    MetadataBindingWorkersBindingKindAISearchNamespace,
    MetadataBindingWorkersBindingKindAnalyticsEngine,
    MetadataBindingWorkersBindingKindAssets,
    MetadataBindingWorkersBindingKindBrowser,
    MetadataBindingWorkersBindingKindD1,
    MetadataBindingWorkersBindingKindDataBlob,
    MetadataBindingWorkersBindingKindDispatchNamespace,
    MetadataBindingWorkersBindingKindDurableObjectNamespace,
    MetadataBindingWorkersBindingKindHyperdrive,
    MetadataBindingWorkersBindingKindInherit,
    MetadataBindingWorkersBindingKindImages,
    MetadataBindingWorkersBindingKindJson,
    MetadataBindingWorkersBindingKindKVNamespace,
    MetadataBindingWorkersBindingKindMedia,
    MetadataBindingWorkersBindingKindMTLSCertificate,
    MetadataBindingWorkersBindingKindPlainText,
    MetadataBindingWorkersBindingKindPipelines,
    MetadataBindingWorkersBindingKindQueue,
    MetadataBindingWorkersBindingKindRatelimit,
    MetadataBindingWorkersBindingKindR2Bucket,
    MetadataBindingWorkersBindingKindSecretText,
    MetadataBindingWorkersBindingKindSendEmail,
    MetadataBindingWorkersBindingKindService,
    MetadataBindingWorkersBindingKindTextBlob,
    MetadataBindingWorkersBindingKindVectorize,
    MetadataBindingWorkersBindingKindVersionMetadata,
    MetadataBindingWorkersBindingKindSecretsStoreSecret,
    MetadataBindingWorkersBindingKindFlagship,
    MetadataBindingWorkersBindingKindSecretKey,
    MetadataBindingWorkersBindingKindWorkflow,
    MetadataBindingWorkersBindingKindWasmModule,
    MetadataBindingWorkersBindingKindVPCService,
    MetadataBindingWorkersBindingKindVPCNetwork,
]


class MetadataLimits(TypedDict, total=False):
    """Limits to apply for this Worker."""

    cpu_ms: int
    """The amount of CPU time this Worker can use in milliseconds."""

    subrequests: int
    """The number of subrequests this Worker can make per request."""


class MetadataMigrationsWorkersMultipleStepMigrations(TypedDict, total=False):
    new_tag: str
    """Tag to set as the latest migration tag."""

    old_tag: str
    """Tag used to verify against the latest migration tag for this Worker.

    If they don't match, the upload is rejected.
    """

    steps: Iterable[MigrationStepParam]
    """Migrations to apply in order."""


MetadataMigrations: TypeAlias = Union[SingleStepMigrationParam, MetadataMigrationsWorkersMultipleStepMigrations]


class MetadataObservabilityLogs(TypedDict, total=False):
    """Log settings for the Worker."""

    enabled: Required[bool]
    """Whether logs are enabled for the Worker."""

    invocation_logs: Required[bool]
    """
    Whether
    [invocation logs](https://developers.cloudflare.com/workers/observability/logs/workers-logs/#invocation-logs)
    are enabled for the Worker.
    """

    destinations: SequenceNotStr[str]
    """A list of destinations where logs will be exported to."""

    head_sampling_rate: Optional[float]
    """The sampling rate for logs. From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1."""

    persist: bool
    """Whether log persistence is enabled for the Worker."""


class MetadataObservabilityTraces(TypedDict, total=False):
    """Trace settings for the Worker."""

    destinations: SequenceNotStr[str]
    """A list of destinations where traces will be exported to."""

    enabled: bool
    """Whether traces are enabled for the Worker."""

    head_sampling_rate: Optional[float]
    """The sampling rate for traces. From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1."""

    persist: bool
    """Whether trace persistence is enabled for the Worker."""


class MetadataObservability(TypedDict, total=False):
    """Observability settings for the Worker."""

    enabled: Required[bool]
    """Whether observability is enabled for the Worker."""

    head_sampling_rate: Optional[float]
    """The sampling rate for incoming requests.

    From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1.
    """

    logs: Optional[MetadataObservabilityLogs]
    """Log settings for the Worker."""

    traces: Optional[MetadataObservabilityTraces]
    """Trace settings for the Worker."""


class MetadataPlacementUnionMember0(TypedDict, total=False):
    mode: Required[Literal["smart"]]
    """
    Enables
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """


class MetadataPlacementUnionMember1(TypedDict, total=False):
    region: Required[str]
    """Cloud region for targeted placement in format 'provider:region'."""


class MetadataPlacementUnionMember2(TypedDict, total=False):
    hostname: Required[str]
    """HTTP hostname for targeted placement."""


class MetadataPlacementUnionMember3(TypedDict, total=False):
    host: Required[str]
    """TCP host and port for targeted placement."""


class MetadataPlacementUnionMember4(TypedDict, total=False):
    mode: Required[Literal["targeted"]]
    """Targeted placement mode."""

    region: Required[str]
    """Cloud region for targeted placement in format 'provider:region'."""


class MetadataPlacementUnionMember5(TypedDict, total=False):
    hostname: Required[str]
    """HTTP hostname for targeted placement."""

    mode: Required[Literal["targeted"]]
    """Targeted placement mode."""


class MetadataPlacementUnionMember6(TypedDict, total=False):
    host: Required[str]
    """TCP host and port for targeted placement."""

    mode: Required[Literal["targeted"]]
    """Targeted placement mode."""


class MetadataPlacementUnionMember7TargetRegion(TypedDict, total=False):
    region: Required[str]
    """Cloud region in format 'provider:region'."""


class MetadataPlacementUnionMember7TargetHostname(TypedDict, total=False):
    hostname: Required[str]
    """HTTP hostname for targeted placement."""


class MetadataPlacementUnionMember7TargetHost(TypedDict, total=False):
    host: Required[str]
    """TCP host:port for targeted placement."""


MetadataPlacementUnionMember7Target: TypeAlias = Union[
    MetadataPlacementUnionMember7TargetRegion,
    MetadataPlacementUnionMember7TargetHostname,
    MetadataPlacementUnionMember7TargetHost,
]


class MetadataPlacementUnionMember7(TypedDict, total=False):
    mode: Required[Literal["targeted"]]
    """Targeted placement mode."""

    target: Required[Iterable[MetadataPlacementUnionMember7Target]]
    """Array of placement targets (currently limited to single target)."""


MetadataPlacement: TypeAlias = Union[
    MetadataPlacementUnionMember0,
    MetadataPlacementUnionMember1,
    MetadataPlacementUnionMember2,
    MetadataPlacementUnionMember3,
    MetadataPlacementUnionMember4,
    MetadataPlacementUnionMember5,
    MetadataPlacementUnionMember6,
    MetadataPlacementUnionMember7,
]


class Metadata(TypedDict, total=False):
    """JSON-encoded metadata about the uploaded parts and Worker configuration."""

    assets: MetadataAssets
    """Configuration for assets within a Worker."""

    bindings: Iterable[MetadataBinding]
    """List of bindings attached to a Worker.

    You can find more about bindings on our docs:
    https://developers.cloudflare.com/workers/configuration/multipart-upload-metadata/#bindings.
    """

    body_part: str
    """Name of the uploaded file that contains the script (e.g.

    the file adding a listener to the `fetch` event). Indicates a
    `service worker syntax` Worker.
    """

    compatibility_date: str
    """Date indicating targeted support in the Workers runtime.

    Backwards incompatible fixes to the runtime following this date will not affect
    this Worker.
    """

    compatibility_flags: SequenceNotStr[str]
    """Flags that enable or disable certain features in the Workers runtime.

    Used to enable upcoming features or opt in or out of specific changes not
    included in a `compatibility_date`.
    """

    keep_assets: bool
    """
    Retain assets which exist for a previously uploaded Worker version; used in lieu
    of providing a completion token.
    """

    keep_bindings: SequenceNotStr[str]
    """List of binding types to keep from previous_upload."""

    limits: MetadataLimits
    """Limits to apply for this Worker."""

    logpush: bool
    """Whether Logpush is turned on for the Worker."""

    main_module: str
    """Name of the uploaded file that contains the main module (e.g.

    the file exporting a `fetch` handler). Indicates a `module syntax` Worker.
    """

    migrations: MetadataMigrations
    """Migrations to apply for Durable Objects associated with this Worker."""

    observability: MetadataObservability
    """Observability settings for the Worker."""

    placement: MetadataPlacement
    """
    Configuration for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    Specify mode='smart' for Smart Placement, or one of region/hostname/host.
    """

    tags: SequenceNotStr[str]
    """List of strings to use as tags for this Worker."""

    tail_consumers: Optional[Iterable[ConsumerScriptParam]]
    """List of Workers that will consume logs from the attached Worker."""

    usage_model: Literal["standard", "bundled", "unbound"]
    """Usage model for the Worker invocations."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["ASPAChangesResponse", "ASNInfo", "ASNInfo_13335", "Change", "ChangeEntry", "Meta"]


class ASNInfo_13335(BaseModel):
    asn: int
    """ASN number."""

    country: str
    """Alpha-2 country code."""

    name: str
    """AS name."""


class ASNInfo(BaseModel):
    api_13335: ASNInfo_13335 = FieldInfo(alias="13335")


class ChangeEntry(BaseModel):
    customer_asn: int = FieldInfo(alias="customerAsn")
    """The customer ASN affected."""

    providers: List[int]

    type: Literal["CustomerAdded", "CustomerRemoved", "ProvidersAdded", "ProvidersRemoved"]


class Change(BaseModel):
    customers_added: int = FieldInfo(alias="customersAdded")
    """Number of new ASPA objects created."""

    customers_removed: int = FieldInfo(alias="customersRemoved")
    """Number of ASPA objects deleted."""

    date: datetime
    """Date of the changes in ISO 8601 format."""

    entries: List[ChangeEntry]

    providers_added: int = FieldInfo(alias="providersAdded")
    """Number of providers added to existing objects."""

    providers_removed: int = FieldInfo(alias="providersRemoved")
    """Number of providers removed from existing objects."""

    total_count: int = FieldInfo(alias="totalCount")
    """Running total of active ASPA objects after this day."""


class Meta(BaseModel):
    data_time: datetime = FieldInfo(alias="dataTime")
    """Timestamp of the underlying data."""

    query_time: datetime = FieldInfo(alias="queryTime")
    """Timestamp when the query was executed."""


class ASPAChangesResponse(BaseModel):
    asn_info: ASNInfo = FieldInfo(alias="asnInfo")

    changes: List[Change]

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["ConfigCreateParams"]


class ConfigCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Defines the Account ID."""

    ips: Required[SequenceNotStr[str]]
    """Defines a list of IP addresses or CIDR blocks to scan.

    The maximum number of total IP addresses allowed is 5000.
    """

    frequency: float
    """Defines the number of days between each scan (0 = One-off scan)."""

    ports: SequenceNotStr[str]
    """Defines a list of ports to scan.

    Valid values are:"default", "all", or a comma-separated list of ports or range
    of ports (e.g. ["1-80", "443"]). "default" scans the 100 most commonly open
    ports.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["OldPublicSchema"]


class OldPublicSchema(BaseModel):
    created_at: datetime

    kind: Literal["openapi_v3"]
    """Kind of schema"""

    name: str
    """Name of the schema"""

    schema_id: str
    """UUID."""

    source: Optional[str] = None
    """Source of the schema"""

    validation_enabled: Optional[bool] = None
    """Flag whether schema is enabled for validation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .view_edit_params import ViewEditParams as ViewEditParams
from .view_list_params import ViewListParams as ViewListParams
from .view_get_response import ViewGetResponse as ViewGetResponse
from .view_create_params import ViewCreateParams as ViewCreateParams
from .view_edit_response import ViewEditResponse as ViewEditResponse
from .view_list_response import ViewListResponse as ViewListResponse
from .view_create_response import ViewCreateResponse as ViewCreateResponse
from .view_delete_response import ViewDeleteResponse as ViewDeleteResponse
